package com.mysema.commons.thumbs;

import java.awt.image.BufferedImage;
import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.IOException;

import javax.imageio.ImageIO;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.drew.imaging.jpeg.JpegMetadataReader;
import com.drew.imaging.jpeg.JpegProcessingException;
import com.drew.metadata.Directory;
import com.drew.metadata.Metadata;
import com.drew.metadata.MetadataException;
import com.drew.metadata.exif.ExifDirectory;

/**
 * @author tiwe
 * 
 */
abstract class AbstractFactory extends ThumbFactory {

    private static final Logger logger = LoggerFactory.getLogger(AbstractFactory.class);

    private static final int TDATA = ExifDirectory.TAG_THUMBNAIL_DATA;

    private static final int TWIDTH = ExifDirectory.TAG_THUMBNAIL_IMAGE_WIDTH;

    private static final int THEIGHT = ExifDirectory.TAG_THUMBNAIL_IMAGE_HEIGHT;

    protected BufferedImage getJPEGThumbnail(File file, int maxWidth,
            int maxHeight) {
        try {
            Metadata metadata = JpegMetadataReader.readMetadata(file);
            if (metadata.containsDirectory(ExifDirectory.class)) {
                Directory dir = metadata.getDirectory(ExifDirectory.class);
                if (dir.containsTag(TDATA)) {
                    if (dir.containsTag(TWIDTH) && dir.containsTag(THEIGHT)) {
                        int twidth = dir.getInt(TWIDTH);
                        int theight = dir.getInt(THEIGHT);
                        if (twidth < maxWidth && theight < maxHeight) {
                            logger.error("thumb too small : " + twidth + "/"
                                    + theight);
                            return null;
                        }
                    }

                    byte[] bytes = dir
                            .getByteArray(ExifDirectory.TAG_THUMBNAIL_DATA);
                    BufferedImage image = ImageIO
                            .read(new ByteArrayInputStream(bytes));
                    if (image.getWidth() < maxWidth
                            && image.getHeight() < maxHeight) {
                        logger.error("thumb too small : " + image.getWidth()
                                + "/" + image.getHeight());
                        return null;
                    } else {
                        logger.error(file.getPath() + " from thumb");
                        return image;
                    }
                }
            }
        } catch (RuntimeException e) {
            logger.error(e.getMessage(), e);
        } catch (MetadataException e) {
            logger.error(e.getMessage(), e);
        } catch (IOException e) {
            logger.error(e.getMessage(), e);
        } catch (JpegProcessingException e) {
            logger.error(e.getMessage(), e);
        }
        return null;
    }

    protected double scaleToFit(double w1, double h1, double w2, double h2) {
        double scale = 1.0D;
        if (w1 > h1) {
            if (w1 > w2) {
                scale = w2 / w1;
            }
            h1 *= scale;
            if (h1 > h2) {
                scale *= h2 / h1;
            }
        } else {
            if (h1 > h2) {
                scale = h2 / h1;
            }
            w1 *= scale;
            if (w1 > w2) {
                scale *= w2 / w1;
            }
        }
        return scale;
    }
}
