package com.mysema.commons.thumbs;

import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.RenderingHints;
import java.awt.geom.AffineTransform;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;

import javax.imageio.ImageIO;

/**
 * @author tiwe
 * 
 */
class DefaultFactory extends AbstractFactory {

    public BufferedImage createThumbnail(File file, int maxWidth, int maxHeight) {
        try {
            BufferedImage image = null;
            String name = file.getName().toLowerCase();
            if (name.endsWith(".jpeg") || name.endsWith(".jpg")) {
                image = getJPEGThumbnail(file, maxWidth, maxHeight);
            }
            if (image == null) {
                image = ImageIO.read(file);
            }
            return createThumbnail(image, maxWidth, maxHeight);
        } catch (IOException e) {
            throw new IllegalArgumentException(e.getMessage(), e);
        }
    }

    public BufferedImage createThumbnail(BufferedImage image, int maxWidth,
            int maxHeight) {
        if (image.getWidth(null) < maxWidth
                && image.getHeight(null) < maxHeight) {
            return image;
        }
        boolean cont = true;
        while (cont) {
            double scale = scaleToFit(image.getWidth(), image.getHeight(),
                    maxWidth, maxHeight);
            int width, height;
            if (scale < 0.5) {
                scale = 0.5;
                width = (int) (image.getWidth() * scale);
                height = (int) (image.getHeight() * scale);
            } else {
                cont = false;
                width = maxWidth;
                height = maxHeight;

                double thumbRatio = (double) width / (double) height;
                double imageRatio = (double) image.getWidth()
                        / (double) image.getHeight();
                if (thumbRatio < imageRatio) {
                    height = (int) (width / imageRatio);
                } else {
                    width = (int) (height * imageRatio);
                }
            }

            AffineTransform transform = AffineTransform.getScaleInstance(scale,
                    scale);

            // create thumbnail
            BufferedImage thumbnail = new BufferedImage(width, height,
                    BufferedImage.TYPE_INT_RGB);
            Graphics2D g2d = thumbnail.createGraphics();

            g2d.setBackground(Color.WHITE);

            g2d.setRenderingHint(RenderingHints.KEY_RENDERING,
                    RenderingHints.VALUE_RENDER_QUALITY);
            g2d.setRenderingHint(RenderingHints.KEY_COLOR_RENDERING,
                    RenderingHints.VALUE_COLOR_RENDER_QUALITY);
            g2d.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
                    RenderingHints.VALUE_ANTIALIAS_ON);
            g2d.setRenderingHint(RenderingHints.KEY_DITHERING,
                    RenderingHints.VALUE_DITHER_ENABLE);
            g2d.setRenderingHint(RenderingHints.KEY_INTERPOLATION,
                    RenderingHints.VALUE_INTERPOLATION_BICUBIC);
            g2d.clearRect(0, 0, width, height);
            g2d.drawRenderedImage(image, transform);
            g2d.dispose();

            image = thumbnail;
        }

        return image;
    }

}
