package com.mysema.commons.thumbs;

import java.awt.Graphics2D;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;

import javax.imageio.ImageIO;

/**
 * @author tiwe
 * 
 */
class FastFactory extends AbstractFactory {

    public BufferedImage createThumbnail(BufferedImage image, int maxWidth,
            int maxHeight) {
        return toThumbnail(image, maxWidth, maxHeight);
    }

    public BufferedImage createThumbnail(File file, int maxWidth, int maxHeight) {
        try {
            BufferedImage image = null;
            String name = file.getName().toLowerCase();
            if (name.endsWith(".jpeg") || name.endsWith(".jpg")) {
                image = getJPEGThumbnail(file, maxWidth, maxHeight);
            }
            if (image == null) {
                image = ImageIO.read(file);
            }
            return toThumbnail(image, maxWidth, maxHeight);
        } catch (IOException e) {
            throw new IllegalArgumentException(e.getMessage(), e);
        }
    }

    private BufferedImage toThumbnail(BufferedImage image, int maxWidth,
            int maxHeight) {
        if (image.getWidth(null) < maxWidth
                && image.getHeight(null) < maxHeight) {
            return image;
        }
        double thumbRatio = (double) maxWidth / (double) maxHeight;
        int imageWidth = image.getWidth(null);
        int imageHeight = image.getHeight(null);
        double imageRatio = (double) imageWidth / (double) imageHeight;
        if (thumbRatio < imageRatio) {
            maxHeight = (int) (maxWidth / imageRatio);
        } else {
            maxWidth = (int) (maxHeight * imageRatio);
        }

        BufferedImage thumbnail = new BufferedImage(maxWidth, maxHeight,
                BufferedImage.TYPE_INT_RGB);
        Graphics2D g2d = thumbnail.createGraphics();
        g2d.drawImage(image, 0, 0, maxWidth, maxHeight, null);
        return thumbnail;
    }

}
