/*
 * Copyright (c) 2009 Mysema Ltd.
 * All rights reserved.
 * 
 */
package com.mysema.commons.thumbs;

import java.awt.image.BufferedImage;
import java.io.File;

/**
 * ThumbFactory generates thumbnails with a given max width and max height. See
 * ThumbFactoryTest for usage examples.
 * 
 * @author tiwe
 * @version $Id$
 */
public abstract class ThumbFactory {

    private static ThumbFactory defaultFactory = new DefaultFactory();

    private static ThumbFactory fastFactory = new FastFactory();

    public static BufferedImage getThumbnail(File file, int maxWidth,
            int maxHeight) {
        return defaultFactory.createThumbnail(file, maxWidth, maxHeight);
    }

    public static BufferedImage getThumbnail(BufferedImage image, int maxWidth,
            int maxHeight) {
        return defaultFactory.createThumbnail(image, maxWidth, maxHeight);
    }

    public static ThumbFactory getDefaultFactory() {
        return defaultFactory;
    }

    public static ThumbFactory getFastFactory() {
        return fastFactory;
    }

    /**
     * Create a thumb for the give file with the given dimensions
     * 
     * @param file
     * @param maxWidth
     * @param maxHeight
     * @return
     */
    public abstract BufferedImage createThumbnail(File file, int maxWidth, int maxHeight);

    /**
     * Create a thumb for the given image with the given dimensions
     * 
     * @param image
     * @param maxWidth
     * @param maxHeight
     * @return
     */
    public abstract BufferedImage createThumbnail(BufferedImage image, int maxWidth, int maxHeight);

}
